--[[
LuCI - Lua Configuration Interface
Copyright 2010-2011 Thinktube.Inc <matsuyama@thinktube.com>

]]--
module("luci.controller.nw-assistance", package.seeall)

function index()
	require("luci.i18n")
	luci.i18n.loadc("nw-assistance")
	local ttu = require "luci.tools.ttutil"

	if ttu.rmr_hw_model() == "RMR-9000" then
		entry({"admin", "network", "nw_assistance"}, call("easy_setup_network"), luci.i18n.translate("nw_assistance_menu"), 99)
	end
end

function easy_setup_network()
	local i18n = luci.i18n.translate
	local uci = luci.model.uci.cursor()
	local ttu = require "luci.tools.ttutil"

	local function is_include(str, tbl)
		for _, v in pairs(tbl) do
			if v == str then
				return true
			end
		end
		return false
	end

	local function decide_number(number)
		local val1
		local val2

		-- determine node number
		if number > 0 and number <= 100 then
			val1 = tostring(number)
			val2 = tostring(number + 100)
		elseif number > 100 and number < 255 then
			val1 = tostring(number)
			val2 = tostring(number - 100)
		else -- fail safe
			val1 = "10"
			val2 = "110"
		end
		return val1, val2
	end

	local function setup_whole_network(gateway, number)
		local node_val, node_val2 = decide_number(number)

		---- get wireless phy device
		-- phy0, phy1 and phy2 is default wireless devices for RMR-9000
		local wdev = {}
		uci:foreach("wireless", "wifi-device",
			function(s)
				if s.phy == "phy0" or s.phy == "phy1" or s.phy == "phy2" then
					wdev[#wdev+1] = s['.name']
				end
			end)

		---- delete all 'wifi-iface' belong to 'phy0', "phy1' and 'phy2'
		uci:delete_all("wireless", "wifi-iface",
			function(s)
				return is_include(s.device, wdev)
			end)

		---- delete all 'interface' if it's 'ADHOC_0', 'ADHOC_1' or 'AP_0'
		uci:delete("network", "ADHOC_0")
		uci:delete("network", "ADHOC_1")
		uci:delete("network", "AP_0")

		---- create new 'wifi-iface'
		uci:section("wireless", "wifi-iface", nil, {
				device     = "RADIO_0",
				network    = "ADHOC_0",
				ifname     = "wlan0",
				mode       = "adhoc",
				ssid       = "mesh01",
				bintval    = "1000",
				basicrates = "1,2,5.5,11,6,12,24",
				encryption = "none"
			})

		uci:section("wireless", "wifi-iface", nil, {
				device     = "RADIO_1",
				network    = "ADHOC_1",
				ifname     = "wlan1",
				mode       = "adhoc",
				ssid       = "mesh06",
				bintval    = "1000",
				basicrates = "1,2,5.5,11,6,12,24",
				encryption = "none"
			})

		uci:section("wireless", "wifi-iface", nil, {
				device     = "RADIO_2",
				network    = "AP_0",
				ifname     = "wlan2",
				mode       = "ap",
				ssid       = "RMR-AP-"..node_val,
				encryption = "psk_mixed",
				key        = "rokkomesh",
			})

		---- update wired network interface
		uci:foreach("network", "interface",
				function(s)
					if s.ifname == "eth1" then
						uci:tset("network", s['.name'], {
							proto   = "static",
							ipaddr  = "192.168."..node_val..".1",
							netmask = "255.255.255.0",
						})
					end
				end)

		---- create wireless interfaces
		uci:section("network", "interface", "ADHOC_0", {
				proto   = "static",
				ipaddr  = "10.0.1."..node_val,
				netmask = "255.255.255.0"
			})

		uci:section("network", "interface", "ADHOC_1", {
				proto   = "static",
				ipaddr  = "10.0.1."..node_val,
				netmask = "255.255.255.0"
			})

		uci:section("network", "interface", "AP_0", {
				proto   = "static",
				ipaddr  = "192.168."..node_val2..".1",
				netmask = "255.255.255.0"
			})

		---- meshcruzer
		uci:tset("meshz", "main", {
				node_name = "RMR-"..node_val,
				ip_address = "10.0.1."..node_val,
				adhoc_subnet = "10.0.1.0/24",
				adhoc_interface = "wlan0 wlan1",
			})

		if gateway == "on" then
			uci:set("meshz", "main", "mesh_gateway", "yes")
		else
			uci:set("meshz", "main", "mesh_gateway", "no")
		end

		-- cleanup external subnet section
		uci:delete_all("meshz", "routing",
			function(s) return true end)

		---- hostname
		local hostname = "RMR-"..node_val
		uci:foreach("system", "system",
			function(s)
				uci:set("system", s['.name'], "hostname", hostname)
			end)

		---- flash
		uci:save("network")
		uci:save("wireless")
		uci:save("meshz")
		uci:save("system")
		uci:commit("network")
		uci:commit("wireless")
		uci:commit("meshz")
		uci:commit("system")
	end

	local gateway = luci.http.formvalue("gateway") or ""
	local nodenum = tonumber(luci.http.formvalue("nodenum")) or 0
	local state = luci.http.formvalue("state") or ""
	local errmes = ""

	-- validate number
	if state and #state > 0 then
		if not nodenum then
			errmes = errmes .. i18n("nwa_setup_invalied1", "Invalied Node Number<br />")
		elseif nodenum > 254 or nodenum < 1 then
			errmes = errmes .. i18n("nwa_setup_invalied2", "Node Number must be larger than 0 and less than 255.<br />")
		end
	end

	if state == "start" and #errmes == 0 then
		local val1, val2 = decide_number(nodenum)
		luci.template.render("nwa/nw-assistance", {
			gateway=gateway, nodenum=nodenum, state=state,
			val1=val1, val2=val2,
		})
	elseif state == "save" and #errmes == 0 then
		setup_whole_network(gateway, nodenum)
		luci.template.render("nwa/nw-assistance", {
			state=state
		})
	elseif state == "reset" then
		luci.template.render("nwa/nw-assistance")
	else
		luci.template.render("nwa/nw-assistance", {errmes=errmes})
	end
end
